/*
    AURORA: the tcpip stack simulator
    part of project aurora by lamagra <access-granted@geocities.com>
    http://lamagra.seKure.de

    -Lamagra: member of b0f http://www.b0f.com
 */

#include "spooflib.c"
#include "readlib.c"
#include <sys/time.h>
#include <stdio.h>
#include <signal.h>

extern char *optarg;
extern int optind;

void usage(char *name)
{
    printf("Usage: %s <source ip> <dest ip> <sport> <dport>\n",name);
    exit(-1);
}

int readsock,sock;
spoofhdr hdr;

void nice_stop(int sig)
{
    char *data;
    long tmp;

    signal(SIGINT,SIG_DFL);    
    printf("Closing connection, press ctrl-c for fast quit\n");

    if(hdr.client)
    {
        tmp = hdr.seqnum;
        hdr.seqnum = hdr.acknum;
        hdr.acknum = tmp;
    }

    send_tcp(sock,hdr.src,hdr.dest,hdr.sport,hdr.dport,TH_FIN|TH_ACK,hdr.seqnum,hdr.acknum + hdr.datasend,1024,0,0);
    while(!(data = gettcpreply(readsock,&hdr))); /* Wait for ACK */
    while(!(data = gettcpreply(readsock,&hdr))); /* Wait for FIN|ACK */
    send_tcp(sock,hdr.src,hdr.dest,hdr.sport,hdr.dport,TH_ACK,hdr.acknum,hdr.seqnum + 1,1024,0,0);
}

void endconnection()
{
    send_tcp(sock,hdr.src,hdr.dest,hdr.sport,hdr.dport,TH_ACK,hdr.acknum,hdr.seqnum + 1,1024,0,0);
    printf("Connection Closed\n");
    exit(0);
}

main(int argc,char **argv)
{
    int ret,tmp,i=0;
    char *data, buf[2048];
    fd_set fdset;

    if(argc < 5) usage(argv[0]);
    
    if((sock = socket(AF_INET,SOCK_RAW,IPPROTO_RAW)) == -1)
    {
        perror("socket");
        exit(-1);
    }

    signal(SIGINT,nice_stop);
    srand(time(NULL));

    memset((char *)&hdr,0,sizeof(spoofhdr));
    
    if((hdr.src = resolve(argv[1])) == -1)
    {
        hdr.src = rand_ip();
    }

   hdr.dest = resolve(argv[2]);
   hdr.seqnum = random();
   hdr.sport = atol(argv[3]);     
   hdr.dport = atol(argv[4]);     

   readsock = init_read(0);

   ret = send_tcp(sock,hdr.src,hdr.dest,hdr.sport,hdr.dport,TH_SYN,hdr.seqnum,hdr.acknum,1024,0,0);
   while(!(data = gettcpreply(readsock,&hdr)));
   if((hdr.flags & TH_RST) || (hdr.flags & TH_FIN) || !(hdr.flags & TH_SYN|TH_ACK)) 
         endconnection();

#ifdef DEBUG   
   printf("syn|ack %lu | %lu\n",hdr.seqnum,hdr.acknum);
#endif 

   ret = send_tcp(sock,hdr.src,hdr.dest,hdr.sport,hdr.dport,TH_ACK,hdr.acknum,hdr.seqnum + 1,1024,0,0);
   tmp = hdr.seqnum;
   hdr.seqnum = hdr.acknum;
   hdr.acknum = tmp + 1;

   while(1)
   {
        FD_ZERO(&fdset);
        FD_SET(0,&fdset);
        FD_SET(readsock,&fdset);
        
        if(select(readsock + 1,&fdset,NULL,NULL,NULL) == -1)
        {
            perror("select");
            exit(-1);
        }

        if(FD_ISSET(0,&fdset))
        {
            memset(buf,0,sizeof(buf));
            read(0,buf,sizeof(buf));

            if(hdr.client) 
            {
                tmp = hdr.seqnum;
                hdr.seqnum = hdr.acknum;
                hdr.acknum = tmp;
            }
            
            ret = send_tcp(sock,hdr.src,hdr.dest,hdr.sport,hdr.dport,TH_PUSH|TH_ACK,hdr.seqnum,hdr.acknum,1024,buf,strlen(buf));
            hdr.datasend = strlen(buf);     
#ifdef DEBUG                 
            printf(" pushed  %lu | %lu\n",hdr.seqnum,hdr.acknum);
#endif
            while(!(data = gettcpreply(readsock,&hdr))); /* Wait for ACK */
            hdr.client = 1;
            if(hdr.flags & TH_PUSH)
            {
                write(fileno(stdout),data,hdr.datasend);
            }
        }

        if(FD_ISSET(readsock,&fdset))
        {
            if(!(data = gettcpreply(readsock,&hdr))) continue;
            hdr.client = 1;
            
            if((hdr.flags & TH_RST) || (hdr.flags & TH_FIN)) 
                endconnection();
                
            if((hdr.flags & TH_ACK) && (hdr.flags & TH_PUSH))
            {
                write(fileno(stdout),data,hdr.datasend);
#ifdef DEBUG                  
                printf(" push|ack %lu | %lu\n",hdr.seqnum,hdr.acknum);
#endif

                tmp         = hdr.seqnum;
                hdr.seqnum  = hdr.acknum;
                hdr.acknum  = tmp + hdr.datasend;
                send_tcp(sock,hdr.src,hdr.dest,hdr.sport,hdr.dport,TH_ACK,hdr.seqnum,hdr.acknum,1024,0,0);
                hdr.datasend = hdr.client = 0;
            }
        }
   }
}
